@@ -15,9 +15,7 @@
  *
  *)
 
-open Sexplib.Std
-
-exception Parse_error of string * string [@@deriving sexp]
+exception Parse_error of string * string
 
 type scope =
 | Point
@@ -27,7 +25,32 @@ type scope =
 | Site
 | Organization
 | Global
-[@@deriving sexp]
+
+let try_with_result fn a =
+  try Ok (fn a)
+  with Parse_error (msg, _) -> Error (`Msg ("Ipaddr: " ^ msg))
+
+let string_of_scope = function
+| Point -> "point"
+| Interface -> "interface"
+| Link -> "link"
+| Admin -> "admin"
+| Site -> "site"
+| Organization -> "organization"
+| Global -> "global"
+
+let scope_of_string = function
+| "point" -> Ok Point
+| "interface" -> Ok Interface
+| "link" -> Ok Link
+| "admin" -> Ok Admin
+| "site" -> Ok Site
+| "organization" -> Ok Organization
+| "global" -> Ok Global
+| s -> Error (`Msg ("unknown scope: " ^ s))
+
+let pp_scope fmt s =
+  Format.pp_print_string fmt (string_of_scope s)
 
 let (~|) = Int32.of_int
 let (|~) = Int32.to_int
@@ -39,7 +62,6 @@ let (>!)  x y = (x >|> y) &&& 0xFF_l
 let (<!)  x y = (x &&& 0xFF_l) <|< y
 
 let need_more x = Parse_error ("not enough data", x)
-let too_much x = Parse_error ("too much data", x)
 
 let char_0 = int_of_char '0'
 let char_a = int_of_char 'a'
@@ -147,7 +169,8 @@ module V4 = struct
     let x = of_string_raw s o in
     expect_end s o;
     x
-  let of_string s = try Some (of_string_exn s) with _ -> None
+
+  let of_string s = try_with_result of_string_exn s
 
   let to_buffer b i =
     Printf.bprintf b "%ld.%ld.%ld.%ld" (i >! 24) (i >! 16) (i >! 8) (i >! 0)
@@ -157,46 +180,41 @@ module V4 = struct
     to_buffer b i;
     Buffer.contents b
 
-  let pp_hum ppf i =
+  let pp ppf i =
     Format.fprintf ppf "%s" (to_string i)
 
-  let sexp_of_t i =
-    Sexplib.Sexp.Atom (to_string i)
+  (* Octets conversion *)
 
-  let t_of_sexp i =
-    match i with
-    | Sexplib.Sexp.Atom i -> of_string_exn i
-    | _ -> raise (Failure "Ipaddr.V4.t: Unexpected non-atom in sexp")
+  let of_octets_exn ?(off=0) bs =
+    try
+      make
+      (Char.code bs.[0 + off])
+      (Char.code bs.[1 + off])
+      (Char.code bs.[2 + off])
+      (Char.code bs.[3 + off])
+    with _ -> raise (need_more bs)
 
-  (* Byte conversion *)
+  let of_octets ?off bs = try_with_result (of_octets_exn ?off) bs
 
-  let of_bytes_raw bs o =
-    make
-      (Char.code bs.[0 + o])
-      (Char.code bs.[1 + o])
-      (Char.code bs.[2 + o])
-      (Char.code bs.[3 + o])
+  let write_octets_exn ?(off=0) i b =
+    try
+      Bytes.set b (0 + off) (Char.chr ((|~) (i >! 24)));
+      Bytes.set b (1 + off) (Char.chr ((|~) (i >! 16)));
+      Bytes.set b (2 + off) (Char.chr ((|~) (i >!  8)));
+      Bytes.set b (3 + off) (Char.chr ((|~) (i >!  0)))
+    with _ -> raise (need_more (Bytes.to_string b))
 
-  let of_bytes_exn bs =
-    let len = String.length bs in
-    if len > 4 then raise (too_much bs);
-    if len < 4 then raise (need_more bs);
-    of_bytes_raw bs 0
+  let write_octets ?off i bs = try_with_result (write_octets_exn ?off i) bs
 
-  let of_bytes bs = try Some (of_bytes_exn bs) with _ -> None
+  let to_octets i =
+    String.init 4 (function
+      | 0 -> Char.chr ((|~) (i >! 24))
+      | 1 -> Char.chr ((|~) (i >! 16))
+      | 2 -> Char.chr ((|~) (i >! 8))
+      | 3 -> Char.chr ((|~) (i >! 0))
+      | _ -> assert false)
 
-  let to_bytes_raw i b o =
-    Bytes.set b (0 + o) (Char.chr ((|~) (i >! 24)));
-    Bytes.set b (1 + o) (Char.chr ((|~) (i >! 16)));
-    Bytes.set b (2 + o) (Char.chr ((|~) (i >!  8)));
-    Bytes.set b (3 + o) (Char.chr ((|~) (i >!  0)))
-
-  let to_bytes i =
-    let b = Bytes.create 4 in
-    to_bytes_raw i b 0;
-    Bytes.to_string b
-
-  (* Int32*)
+  (* Int32 *)
   let of_int32 i = i
   let to_int32 i = i
 
@@ -204,8 +222,8 @@ module V4 = struct
   let of_int16 (a,b) = (~| a <|< 16) ||| (~| b)
   let to_int16 a = ((|~) (a >|> 16), (|~) (a &&& 0xFF_FF_l))
 
-  (** MAC *)
-  (** {{:http://tools.ietf.org/html/rfc1112#section-6.2}RFC 1112}. *)
+  (* MAC *)
+  (* {{:http://tools.ietf.org/html/rfc1112#section-6.2}RFC 1112}. *)
   let multicast_to_mac i =
     let macb = Bytes.create 6 in
     Bytes.set macb 0 (Char.chr 0x01);
@@ -214,18 +232,39 @@ module V4 = struct
     Bytes.set macb 3 (Char.chr ((|~) (i >|> 16 &&& 0x7F_l)));
     Bytes.set macb 4 (Char.chr ((|~) (i >! 8)));
     Bytes.set macb 5 (Char.chr ((|~) (i >! 0)));
-    Macaddr.of_bytes_exn (Bytes.to_string macb)
+    Macaddr.of_octets_exn (Bytes.to_string macb)
 
   (* Host *)
-  let to_domain_name i = [
-    Int32.to_string (i >!  0);
-    Int32.to_string (i >!  8);
-    Int32.to_string (i >! 16);
-    Int32.to_string (i >! 24);
-    "in-addr";
-    "arpa";
-    "";
-  ]
+  let to_domain_name i =
+    let name = [
+      Int32.to_string (i >!  0);
+      Int32.to_string (i >!  8);
+      Int32.to_string (i >! 16);
+      Int32.to_string (i >! 24);
+      "in-addr";
+      "arpa" ]
+    in
+    Domain_name.(host_exn (of_strings_exn name))
+
+  let of_domain_name n =
+    match Domain_name.to_strings n with
+    | [ a ; b ; c ; d ; in_addr ; arpa ] when
+        Domain_name.(equal_label arpa "arpa" && equal_label in_addr "in-addr") ->
+      begin
+        let conv bits data =
+          let i = Int32.of_int (parse_dec_int data (ref 0)) in
+          if i > 0xFFl then
+            raise (Parse_error ("label with a too big number", data))
+          else
+            i <! bits
+        in
+        try
+          let ( + ) = Int32.add in
+          Some ((conv 0 a) + (conv 8 b) + (conv 16 c) + (conv 24 d))
+        with
+        | Parse_error _ -> None
+      end
+    | _ -> None
 
   (* constant *)
 
@@ -237,8 +276,8 @@ module V4 = struct
   let routers     = make 224   0   0   2
 
   module Prefix = struct
-    type addr = t [@@deriving sexp]
-    type t = addr * int [@@deriving sexp]
+    type addr = t
+    type t = addr * int
 
     let compare (pre,sz) (pre',sz') =
       let c = compare pre pre' in
@@ -278,12 +317,12 @@ module V4 = struct
 
     let of_string_exn s = let (p,quad) = _of_string_exn s in make p quad
 
-    let of_string s = try Some (of_string_exn s) with _ -> None
+    let of_string s = try_with_result of_string_exn s
 
     let of_address_string_exn s =
       let (p,quad) = _of_string_exn s in (make p quad, quad)
 
-    let of_address_string s = try Some (of_address_string_exn s) with _ -> None
+    let of_address_string s = try_with_result of_address_string_exn s
 
     let of_netmask nm addr =
       let rec find_greatest_one bits i =
@@ -302,7 +341,7 @@ module V4 = struct
       to_buffer b subnet;
       Buffer.contents b
 
-    let pp_hum ppf i =
+    let pp ppf i =
       Format.fprintf ppf "%s" (to_string i)
 
     let to_address_buffer buf ((_,sz) as subnet) addr =
@@ -339,8 +378,8 @@ module V4 = struct
     ]
 
     let broadcast (pre,sz) = pre ||| (0x0_FF_FF_FF_FF_l >|> sz)
-    let network (pre,sz) = pre
-    let bits (pre,sz) = sz
+    let network (pre,_) = pre
+    let bits (_,sz) = sz
     let netmask subnet = mask (bits subnet)
   end
 
@@ -396,21 +435,11 @@ module B128 = struct
     in
     (a,b,c,d,e,f,g,h)
 
-  let to_bytes_raw (a,b,c,d) byte o =
-    V4.to_bytes_raw a byte (o+0);
-    V4.to_bytes_raw b byte (o+4);
-    V4.to_bytes_raw c byte (o+8);
-    V4.to_bytes_raw d byte (o+12)
-
-  let of_bytes_exn bs = (* TODO : from cstruct *)
-    let len = String.length bs in
-    if len > 16 then raise (too_much bs);
-    if len < 16 then raise (need_more bs);
-    let hihi = V4.of_bytes_raw bs 0 in
-    let hilo = V4.of_bytes_raw bs 4 in
-    let lohi = V4.of_bytes_raw bs 8 in
-    let lolo = V4.of_bytes_raw bs 12 in
-    of_int32 (hihi, hilo, lohi, lolo)
+  let write_octets_exn ?(off=0) (a,b,c,d) byte =
+    V4.write_octets_exn ~off a byte;
+    V4.write_octets_exn ~off:(off+4) b byte;
+    V4.write_octets_exn ~off:(off+8) c byte;
+    V4.write_octets_exn ~off:(off+12) d byte
 
   let compare (a1,b1,c1,d1) (a2,b2,c2,d2) =
     match V4.compare a1 a2 with
@@ -442,9 +471,10 @@ module V6 = struct
   let parse_ipv6 s i =
     let compressed = ref false in (* :: *)
     let len = String.length s in
-    if len < !i + 2 then (raise (need_more s));
+    if len < !i + 1 then (raise (need_more s));
     let use_bracket = s.[!i] = '[';  in
     if use_bracket then incr i;
+    if len < !i + 2 then (raise (need_more s));
     (* check if it starts with :: *)
     let l =
       if s.[!i] = ':' then begin
@@ -543,16 +573,16 @@ module V6 = struct
     expect_end s o;
     x
 
-  let of_string s = try Some (of_string_exn s) with _ -> None
+  let of_string s = try_with_result of_string_exn s
 
   (* http://tools.ietf.org/html/rfc5952 *)
-  let to_buffer ?(v4=false) buf addr =
+  let to_buffer buf addr =
 
     let (a,b,c,d,e,f,g,h) as comp = to_int16 addr in
 
     let v4 = match comp with
       | (0,0,0,0,0,0xffff,_,_) -> true
-      | _ -> v4
+      | _ -> false
     in
 
     let rec loop elide zeros acc = function
@@ -591,45 +621,34 @@ module V6 = struct
       | [] -> ()
     in fill (List.rev lrev)
 
-  let to_string ?v4 l =
+  let to_string l =
     let buf = Buffer.create 39 in
-    to_buffer ?v4 buf l;
+    to_buffer buf l;
     Buffer.contents buf
 
-  let pp_hum ppf i =
+  let pp ppf i =
     Format.fprintf ppf "%s" (to_string i)
 
-  let sexp_of_t i =
-    Sexplib.Sexp.Atom (to_string i)
-
-  let t_of_sexp i =
-    match i with
-    | Sexplib.Sexp.Atom i -> of_string_exn i
-    | _ -> raise (Failure "Ipaddr.V6.t: Unexpected non-atom in sexp")
-
   (* byte conversion *)
 
-  let of_bytes_raw bs o = (* TODO : from cstruct *)
-    let hihi = V4.of_bytes_raw bs (o + 0) in
-    let hilo = V4.of_bytes_raw bs (o + 4) in
-    let lohi = V4.of_bytes_raw bs (o + 8) in
-    let lolo = V4.of_bytes_raw bs (o + 12) in
+  let of_octets_exn ?(off=0) bs = (* TODO : from cstruct *)
+    let hihi = V4.of_octets_exn ~off bs in
+    let hilo = V4.of_octets_exn ~off:(off+4) bs in
+    let lohi = V4.of_octets_exn ~off:(off+8) bs in
+    let lolo = V4.of_octets_exn ~off:(off+12) bs in
     of_int32 (hihi, hilo, lohi, lolo)
 
-  let of_bytes_exn bs = (* TODO : from cstruct *)
-    let len = String.length bs in
-    if len > 16 then raise (too_much bs);
-    if len < 16 then raise (need_more bs);
-    of_bytes_raw bs 0
+  let of_octets ?off bs = try_with_result (of_octets_exn ?off) bs
 
-  let of_bytes bs = try Some (of_bytes_exn bs) with _ -> None
-  let to_bytes i =
-    let bs = Bytes.create 16 in
-    to_bytes_raw i bs 0;
-    Bytes.to_string bs
+  let write_octets ?off i bs = try_with_result (write_octets_exn ?off i) bs
 
-  (** MAC *)
-  (** {{:https://tools.ietf.org/html/rfc2464#section-7}RFC 2464}. *)
+  let to_octets i =
+    let b = Bytes.create 16 in
+    write_octets_exn i b;
+    Bytes.to_string b
+
+  (* MAC *)
+  (* {{:https://tools.ietf.org/html/rfc2464#section-7}RFC 2464}. *)
   let multicast_to_mac i =
     let (_,_,_,i) = to_int32 i in
     let macb = Bytes.create 6 in
@@ -639,46 +658,81 @@ module V6 = struct
     Bytes.set macb 3 (Char.chr ((|~) (i >! 16)));
     Bytes.set macb 4 (Char.chr ((|~) (i >! 8)));
     Bytes.set macb 5 (Char.chr ((|~) (i >! 0)));
-    Macaddr.of_bytes_exn (Bytes.to_string macb)
+    Macaddr.of_octets_exn (Bytes.to_string macb)
 
   (* Host *)
-  let to_domain_name (a,b,c,d) = Printf.([
-    hex_string_of_int32 ((d >|>  0) &&& 0xF_l);
-    hex_string_of_int32 ((d >|>  4) &&& 0xF_l);
-    hex_string_of_int32 ((d >|>  8) &&& 0xF_l);
-    hex_string_of_int32 ((d >|> 12) &&& 0xF_l);
-    hex_string_of_int32 ((d >|> 16) &&& 0xF_l);
-    hex_string_of_int32 ((d >|> 20) &&& 0xF_l);
-    hex_string_of_int32 ((d >|> 24) &&& 0xF_l);
-    hex_string_of_int32 ((d >|> 28) &&& 0xF_l);
-    hex_string_of_int32 ((c >|>  0) &&& 0xF_l);
-    hex_string_of_int32 ((c >|>  4) &&& 0xF_l);
-    hex_string_of_int32 ((c >|>  8) &&& 0xF_l);
-    hex_string_of_int32 ((c >|> 12) &&& 0xF_l);
-    hex_string_of_int32 ((c >|> 16) &&& 0xF_l);
-    hex_string_of_int32 ((c >|> 20) &&& 0xF_l);
-    hex_string_of_int32 ((c >|> 24) &&& 0xF_l);
-    hex_string_of_int32 ((c >|> 28) &&& 0xF_l);
-    hex_string_of_int32 ((b >|>  0) &&& 0xF_l);
-    hex_string_of_int32 ((b >|>  4) &&& 0xF_l);
-    hex_string_of_int32 ((b >|>  8) &&& 0xF_l);
-    hex_string_of_int32 ((b >|> 12) &&& 0xF_l);
-    hex_string_of_int32 ((b >|> 16) &&& 0xF_l);
-    hex_string_of_int32 ((b >|> 20) &&& 0xF_l);
-    hex_string_of_int32 ((b >|> 24) &&& 0xF_l);
-    hex_string_of_int32 ((b >|> 28) &&& 0xF_l);
-    hex_string_of_int32 ((a >|>  0) &&& 0xF_l);
-    hex_string_of_int32 ((a >|>  4) &&& 0xF_l);
-    hex_string_of_int32 ((a >|>  8) &&& 0xF_l);
-    hex_string_of_int32 ((a >|> 12) &&& 0xF_l);
-    hex_string_of_int32 ((a >|> 16) &&& 0xF_l);
-    hex_string_of_int32 ((a >|> 20) &&& 0xF_l);
-    hex_string_of_int32 ((a >|> 24) &&& 0xF_l);
-    hex_string_of_int32 ((a >|> 28) &&& 0xF_l);
-    "ip6";
-    "arpa";
-    "";
-  ])
+  let to_domain_name (a,b,c,d) =
+    let name = [
+      hex_string_of_int32 ((d >|>  0) &&& 0xF_l);
+      hex_string_of_int32 ((d >|>  4) &&& 0xF_l);
+      hex_string_of_int32 ((d >|>  8) &&& 0xF_l);
+      hex_string_of_int32 ((d >|> 12) &&& 0xF_l);
+      hex_string_of_int32 ((d >|> 16) &&& 0xF_l);
+      hex_string_of_int32 ((d >|> 20) &&& 0xF_l);
+      hex_string_of_int32 ((d >|> 24) &&& 0xF_l);
+      hex_string_of_int32 ((d >|> 28) &&& 0xF_l);
+      hex_string_of_int32 ((c >|>  0) &&& 0xF_l);
+      hex_string_of_int32 ((c >|>  4) &&& 0xF_l);
+      hex_string_of_int32 ((c >|>  8) &&& 0xF_l);
+      hex_string_of_int32 ((c >|> 12) &&& 0xF_l);
+      hex_string_of_int32 ((c >|> 16) &&& 0xF_l);
+      hex_string_of_int32 ((c >|> 20) &&& 0xF_l);
+      hex_string_of_int32 ((c >|> 24) &&& 0xF_l);
+      hex_string_of_int32 ((c >|> 28) &&& 0xF_l);
+      hex_string_of_int32 ((b >|>  0) &&& 0xF_l);
+      hex_string_of_int32 ((b >|>  4) &&& 0xF_l);
+      hex_string_of_int32 ((b >|>  8) &&& 0xF_l);
+      hex_string_of_int32 ((b >|> 12) &&& 0xF_l);
+      hex_string_of_int32 ((b >|> 16) &&& 0xF_l);
+      hex_string_of_int32 ((b >|> 20) &&& 0xF_l);
+      hex_string_of_int32 ((b >|> 24) &&& 0xF_l);
+      hex_string_of_int32 ((b >|> 28) &&& 0xF_l);
+      hex_string_of_int32 ((a >|>  0) &&& 0xF_l);
+      hex_string_of_int32 ((a >|>  4) &&& 0xF_l);
+      hex_string_of_int32 ((a >|>  8) &&& 0xF_l);
+      hex_string_of_int32 ((a >|> 12) &&& 0xF_l);
+      hex_string_of_int32 ((a >|> 16) &&& 0xF_l);
+      hex_string_of_int32 ((a >|> 20) &&& 0xF_l);
+      hex_string_of_int32 ((a >|> 24) &&& 0xF_l);
+      hex_string_of_int32 ((a >|> 28) &&& 0xF_l);
+      "ip6";
+      "arpa"
+    ]
+    in
+    Domain_name.(host_exn (of_strings_exn name))
+
+  let of_domain_name n =
+    let open Domain_name in
+    if count_labels n = 34 then
+      let ip6 = get_label_exn n 32 and arpa = get_label_exn n 33 in
+      if equal_label ip6 "ip6" && equal_label arpa "arpa" then
+        let rev = true in
+        let n' = drop_label_exn ~rev ~amount:2 n in
+        let d = drop_label_exn ~rev ~amount:24 n'
+        and c = drop_label_exn ~amount:8 (drop_label_exn ~rev ~amount:16 n')
+        and b = drop_label_exn ~amount:16 (drop_label_exn ~rev ~amount:8 n')
+        and a = drop_label_exn ~amount:24 n'
+        in
+        let t b d =
+          let v = Int32.of_int (parse_hex_int d (ref 0)) in
+          if v > 0xFl then
+            raise (Parse_error ("number in label too big", d))
+          else
+            v <|< b
+        in
+        let f d =
+          List.fold_left (fun (acc, b) d -> Int32.add acc (t b d), b + 4)
+            (0l, 0) (to_strings d)
+        in
+        try
+          let a', _ = f a and b', _ = f b and c', _ = f c and d', _ = f d in
+          Some (a', b', c', d')
+        with
+        | Parse_error _ -> None
+      else
+        None
+    else
+      None
 
   (* constant *)
 
@@ -691,8 +745,8 @@ module V6 = struct
   let site_routers      = make 0xff05 0 0 0 0 0 0 2
 
   module Prefix = struct
-    type addr = t [@@deriving sexp]
-    type t = addr * int [@@deriving sexp]
+    type addr = t
+    type t = addr * int
 
     let compare (pre,sz) (pre',sz') =
       let c = compare pre pre' in
@@ -700,7 +754,7 @@ module V6 = struct
 
     let ip = make
 
-    let full =
+    let _full =
       let f = 0x0_FFFF_FFFF_l in
       f,f,f,f
 
@@ -735,12 +789,12 @@ module V6 = struct
 
     let of_string_exn s = let (p,v6) = _of_string_exn s in make p v6
 
-    let of_string s = try Some (of_string_exn s) with _ -> None
+    let of_string s = try_with_result of_string_exn s
 
     let of_address_string_exn s =
       let (p,v6) = _of_string_exn s in (make p v6, v6)
 
-    let of_address_string s = try Some (of_address_string_exn s) with _ -> None
+    let of_address_string s = try_with_result of_address_string_exn s
 
     let of_netmask nm addr =
       make (match nm with
@@ -753,14 +807,14 @@ module V6 = struct
       ) addr
 
     let to_buffer buf (pre,sz) =
-      Printf.bprintf buf "%a/%d" (to_buffer ~v4:false) pre sz
+      Printf.bprintf buf "%a/%d" to_buffer pre sz
 
     let to_string subnet =
       let buf = Buffer.create 43 in
       to_buffer buf subnet;
       Buffer.contents buf
 
-    let pp_hum ppf i =
+    let pp ppf i =
       Format.fprintf ppf "%s" (to_string i)
 
     let to_address_buffer buf ((_,sz) as subnet) addr =
@@ -788,8 +842,8 @@ module V6 = struct
     let noneui64_interface  = make   3 (ip 0x0000 0 0 0 0 0 0 0)
     let solicited_node      = make 104 (ip 0xff02 0 0 0 0 1 0xff00 0)
 
-    let network (pre,sz) = pre
-    let bits (pre,sz) = sz
+    let network (pre,_) = pre
+    let bits (_,sz) = sz
     let netmask subnet = mask (bits subnet)
   end
 
@@ -820,7 +874,7 @@ module V6 = struct
   let link_address_of_mac =
     let c b i = Char.code (String.get b i) in
     fun mac ->
-      let bmac = Macaddr.to_bytes mac in
+      let bmac = Macaddr.to_octets mac in
       let c_0 = c bmac 0 lxor 2 in
       let addr = make 0 0 0 0
         (c_0      lsl 8 + c bmac 1)
@@ -835,8 +889,8 @@ module V6 = struct
   let is_private i = (scope i) <> Global
 end
 
-type ('v4,'v6) v4v6 = V4 of 'v4 | V6 of 'v6 [@@deriving sexp]
-type t = (V4.t,V6.t) v4v6 [@@deriving sexp]
+type ('v4,'v6) v4v6 = V4 of 'v4 | V6 of 'v6
+type t = (V4.t,V6.t) v4v6
 
 let compare a b = match a,b with
   | V4 a, V4 b -> V4.compare a b
@@ -852,7 +906,7 @@ let to_buffer buf = function
   | V4 x -> V4.to_buffer buf x
   | V6 x -> V6.to_buffer buf x
 
-let pp_hum ppf i =
+let pp ppf i =
       Format.fprintf ppf "%s" (to_string i)
 
 let of_string_raw s offset =
@@ -874,7 +928,7 @@ let of_string_raw s offset =
 
 let of_string_exn s = of_string_raw s (ref 0)
 
-let of_string s = try Some (of_string_exn s) with _ -> None
+let of_string s = try_with_result of_string_exn s
 
 let v6_of_v4 v4 =
   V6.(Prefix.(network_address ipv4_mapped (of_int32 (0l,0l,0l,v4))))
@@ -910,13 +964,27 @@ let to_domain_name = function
   | V4 v4 -> V4.to_domain_name v4
   | V6 v6 -> V6.to_domain_name v6
 
+let of_domain_name n =
+  match Domain_name.count_labels n with
+  | 6 ->
+    begin match V4.of_domain_name n with
+      | None -> None
+      | Some x -> Some (V4 x)
+    end
+  | 34 ->
+    begin match V6.of_domain_name n with
+      | None -> None
+      | Some x -> Some (V6 x)
+    end
+  | _ -> None
+
 module Prefix = struct
   module Addr = struct
     let to_v6 = to_v6
   end
 
-  type addr = t [@@deriving sexp]
-  type t = (V4.Prefix.t,V6.Prefix.t) v4v6 [@@deriving sexp]
+  type addr = t
+  type t = (V4.Prefix.t,V6.Prefix.t) v4v6
 
   let compare a b = match a,b with
     | V4 a , V4 b -> V4.Prefix.compare a b
@@ -943,7 +1011,7 @@ module Prefix = struct
 
   let of_string_exn s = of_string_raw s (ref 0)
 
-  let of_string s = try Some (of_string_exn s) with _ -> None
+  let of_string s = try_with_result of_string_exn s
 
   let v6_of_v4 v4 = V6.Prefix.make
     (96 + V4.Prefix.bits v4)
@@ -982,6 +1050,6 @@ module Prefix = struct
     | V4 p -> V4 (V4.Prefix.netmask p)
     | V6 p -> V6 (V6.Prefix.netmask p)
 
-  let pp_hum ppf i =
+  let pp ppf i =
     Format.fprintf ppf "%s" (to_string i)
 end
