@@ -1,4 +1,5 @@
 (*
+ * Copyright (c) 2019 Anil Madhavapeddy <anil@recoil.org>
  * Copyright (c) 2013-2015 David Sheets <sheets@alum.mit.edu>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -15,10 +16,14 @@
  *
  *)
 
-(** A library for manipulation of IP address representations. *)
+(** A library for manipulation of IP address representations.
 
-(** Raised when parsing of IP address syntax fails. *)
-exception Parse_error of string * string [@@deriving sexp]
+    {e %%VERSION%% - {{:%%PKG_HOMEPAGE%% }homepage}} *)
+
+(** [Parse_error (err,packet)] is raised when parsing of the IP
+    address syntax fails. [err] contains a human-readable error
+    and [packet] is the original octet list that failed to parse. *)
+exception Parse_error of string * string
 
 (** Type of ordered address scope classifications *)
 type scope =
@@ -29,65 +34,86 @@ type scope =
 | Site
 | Organization
 | Global
-[@@deriving sexp]
+
+(** [string_of_scope scope] returns a human-readable representation
+  of {!scope}. *)
+val string_of_scope : scope -> string
+
+(** [scope_of_string s] returns a {!scope} from a string representation
+  of [s].  Valid string values for [s] can be obtained via {!string_of_scope}. *)
+val scope_of_string : string -> (scope, [> `Msg of string]) result
+
+(** [pp_scope fmt scope] outputs a human-readable representation
+  of {!scope} to the [fmt] formatter. *)
+val pp_scope : Format.formatter -> scope -> unit [@@ocaml.toplevel_printer]
 
 (** A collection of functions for IPv4 addresses. *)
 module V4 : sig
   (** Type of the internet protocol v4 address of a host *)
-  type t [@@deriving sexp]
-  val compare : t -> t -> int
+  type t
 
   (** Converts the low bytes of four int values into an abstract {! V4.t }. *)
   val make : int -> int -> int -> int -> t
 
-  (** {3 Text string conversion} *)
+  (** {3 Text string conversion}
+      These manipulate human-readable IPv4 addresses (for example [192.168.1.2]). *)
 
-  (** [of_string_exn ipv4_string] is the address represented
-      by [ipv4_string]. Raises [Parse_error] if [ipv4_string] is not a
-      valid representation of an IPv4 address. *)
+  (** [of_string s] is the address {!t} represented by the human-readable IPv4
+      address [s]. Returns a human-readable error string if parsing failed. *)
+  val of_string : string -> (t, [> `Msg of string ]) result
+
+  (** [of_string_exn s] is the address {!t} represented as a human-readable IPv4
+      address [s]. Raises {!Parse_error} if [s] is not exactly 4 bytes long. *)
   val of_string_exn : string -> t
 
-  (** Same as [of_string_exn] but returns an option type instead of raising
-      an exception. *)
-  val of_string : string -> t option
-
-  (** Same as [of_string_exn] but takes as an extra argument the offset
-      into the string for reading. *)
+  (** [of_string_raw s off] acts as {!of_string_exn} but takes as an extra
+      argument the offset into the string for reading. [off] will be
+      mutated to an unspecified value during the function call. [s] will
+      a {!Parse_error} exception if it is an invalid or truncated IP address. *)
   val of_string_raw : string -> int ref -> t
 
   (** [to_string ipv4] is the dotted decimal string representation
-      of [ipv4], i.e. XXX.XX.X.XXX. *)
+      of [ipv4], i.e. [XXX.XX.X.XXX]. *)
   val to_string : t -> string
 
   (** [to_buffer buf ipv4] writes the string representation of [ipv4] into the
       buffer [buf]. *)
   val to_buffer : Buffer.t -> t -> unit
 
-  (** [pp_hum f ipv4] outputs a human-readable representation of [ipv4] to
+  (** [pp f ipv4] outputs a human-readable representation of [ipv4] to
       the formatter [f]. *)
-  val pp_hum : Format.formatter -> t -> unit
+  val pp : Format.formatter -> t -> unit [@@ocaml.toplevel_printer]
 
-  (** {3 Bytestring conversion} *)
+  (** {3 Octets conversion}
+      These manipulate IPv4 addresses represented as a sequence of
+      four bytes. (e.g for example [0xc0a80102] will be the representation
+      of the human-readable [192.168.1.2] address. *)
 
-  (** [of_bytes_exn ipv4_octets] is the address represented
-      by [ipv4_octets]. Raises [Parse_error] if [ipv4_octets] is not a
-      valid representation of an IPv4 address. *)
-  val of_bytes_exn : string -> t
+  (** [of_octets ?off s] is the address {!t} represented by the IPv4 octets
+      represented by [s] starting from offset [off] within the string.
+      [s] must be at least [off+4] bytes long.
+      Returns a human-readable error string if parsing fails.
+      [off] defaults to 0. *)
+  val of_octets : ?off:int -> string -> (t, [> `Msg of string ]) result
 
-  (** Same as [of_bytes_exn] but returns an option type instead of raising
-      an exception. *)
-  val of_bytes : string -> t option
+  (** [of_octets_exn ipv4_octets] is the IPv4 address represented
+      by [ipv4_octets] starting from offset [off] within the string.
+      Raises {!Parse_error} if [ipv4_octets] is not at least [off+4] bytes long. 
+      [off] defaults to 0. *)
+  val of_octets_exn : ?off:int -> string -> t
 
-  (** Same as [of_bytes_exn] but take an extra paramenter, the offset into
-      the bytes for reading. *)
-  val of_bytes_raw : string -> int -> t
+  (** [write_octets ?off ipv4 b] writes the [ipv4] as octets to [b] starting
+       from offset [off]. [b] must be at least [off+4] long or an error is
+       returned. *)
+  val write_octets : ?off:int -> t -> bytes -> (unit, [> `Msg of string ]) result
 
-  (** [to_bytes ipv4] is a string of length 4 encoding [ipv4]. *)
-  val to_bytes : t -> string
+  (** [write_octets_exn ?off ipv4 b] writes the [ipv4] as octets to [b] starting
+       from offset [off]. [b] must be at least [off+4] long or a
+       {!Parse_error} is raised. *)
+  val write_octets_exn : ?off:int -> t -> bytes -> unit
 
-  (** [to_bytes_raw ipv4 bytes offset] writes the 4 byte encoding of [ipv4]
-      into [bytes] at offset [offset]. *)
-  val to_bytes_raw : t -> Bytes.t -> int -> unit
+  (** [to_octets ipv4] returns the 4 bytes representing the [ipv4] octets. *)
+  val to_octets : t -> string
 
   (** {3 Int conversion} *)
 
@@ -115,8 +141,12 @@ module V4 : sig
   (** {3 Host conversion} *)
 
   (** [to_domain_name ipv4] is the domain name label list for reverse
-      lookups of [ipv4]. This includes the [.in-addr.arpa.] suffix. *)
-  val to_domain_name : t -> string list
+      lookups of [ipv4]. This includes the [.in-addr.arpa] suffix. *)
+  val to_domain_name : t -> [ `host ] Domain_name.t
+
+  (** [of_domain_name name] is [Some t] if the [name] has an [.in-addr.arpa]
+      suffix, and an IPv4 address prefixed. *)
+  val of_domain_name : 'a Domain_name.t -> t option
 
   (** {3 Common addresses} *)
 
@@ -140,12 +170,10 @@ module V4 : sig
 
   (** A module for manipulating IPv4 network prefixes. *)
   module Prefix : sig
-    type addr = t [@@deriving sexp]
+    type addr = t
 
     (** Type of a internet protocol subnet *)
-    type t [@@deriving sexp]
-
-    val compare : t -> t -> int
+    type t
 
     (** [mask n] is the pseudo-address of an [n] bit subnet mask. *)
     val mask : int -> addr
@@ -158,35 +186,37 @@ module V4 : sig
         See <http://tools.ietf.org/html/rfc4291#section-2.3>. *)
     val network_address : t -> addr -> addr
 
+    (** [of_string cidr] is the subnet prefix represented by the CIDR
+        string, [cidr]. Returns a human-readable parsing error message
+        if [cidr] is not a valid representation of a CIDR notation routing
+        prefix. *)
+    val of_string : string ->  (t, [> `Msg of string ]) result
+
     (** [of_string_exn cidr] is the subnet prefix represented by the CIDR
         string, [cidr]. Raises [Parse_error] if [cidr] is not a valid
         representation of a CIDR notation routing prefix. *)
     val of_string_exn : string -> t
 
-    (** Same as [of_string_exn] but returns an option type instead of raising
-        an exception. *)
-    val of_string : string -> t option
-
-    (** Same as [of_string_exn] but takes as an extra argument the offset
+    (** Same as {!of_string_exn} but takes as an extra argument the offset
         into the string for reading. *)
     val of_string_raw : string -> int ref -> t
 
     (** [to_string prefix] is the CIDR notation string representation
-        of [prefix], i.e. XXX.XX.X.XXX/XX. *)
+        of [prefix], i.e. [XXX.XX.X.XXX/XX]. *)
     val to_string : t -> string
 
-    (** [pp_hum f prefix] outputs a human-readable representation of [prefix]
+    (** [pp f prefix] outputs a human-readable representation of [prefix]
         to the formatter [f]. *)
-    val pp_hum : Format.formatter -> t -> unit
+    val pp : Format.formatter -> t -> unit [@@ocaml.toplevel_printer]
 
     (** [of_address_string_exn cidr_addr] is the address and prefix
-        represented by [cidr_addr]. Raises [Parse_error] if [cidr_addr] is not
+        represented by [cidr_addr]. Raises {!Parse_error} if [cidr_addr] is not
         a valid representation of a CIDR-scoped address. *)
     val of_address_string_exn : string -> t * addr
 
-    (** Same as [of_address_string_exn] but returns an option type instead of
+    (** Same as {!of_address_string_exn} but returns a result type instead of
         raising an exception. *)
-    val of_address_string : string -> (t * addr) option
+    val of_address_string : string -> (t * addr, [> `Msg of string ]) result
 
     (** [to_address_string prefix addr] is the network address
         constructed from [prefix] and [addr]. *)
@@ -281,8 +311,7 @@ end
 (** A collection of functions for IPv6 addresses. *)
 module V6 : sig
   (** Type of the internet protocol v6 address of a host *)
-  type t [@@deriving sexp]
-  val compare : t -> t -> int
+  type t
 
   (** Converts the low bytes of eight int values into an abstract
       {! V6.t }. *)
@@ -291,67 +320,73 @@ module V6 : sig
   (** {3 Text string conversion} *)
 
   (** [of_string_exn ipv6_string] is the address represented
-      by [ipv6_string]. Raises [Parse_error] if [ipv6_string] is not a
+      by [ipv6_string]. Raises {!Parse_error} if [ipv6_string] is not a
       valid representation of an IPv6 address. *)
   val of_string_exn : string -> t
 
   (** Same as [of_string_exn] but returns an option type instead of raising
       an exception. *)
-  val of_string : string -> t option
+  val of_string : string -> (t, [> `Msg of string ]) result
 
   (** Same as [of_string_exn] but takes as an extra argument the offset into
       the string for reading. *)
   val of_string_raw : string -> int ref -> t
 
   (** [to_string ipv6] is the string representation of [ipv6],
-      i.e. XXX:XX:X::XXX:XX. *)
-  val to_string : ?v4:bool -> t -> string
+      i.e. [XXX:XX:X::XXX:XX]. *)
+  val to_string : t -> string
 
   (** [to_buffer buf ipv6] writes the string representation of [ipv6] into the
       buffer [buf]. *)
-  val to_buffer : ?v4:bool -> Buffer.t -> t -> unit
+  val to_buffer : Buffer.t -> t -> unit
 
-  (** [pp_hum f ipv6] outputs a human-readable representation of [ipv6] to
+  (** [pp f ipv6] outputs a human-readable representation of [ipv6] to
       the formatter [f]. *)
-  val pp_hum : Format.formatter -> t -> unit
+  val pp : Format.formatter -> t -> unit [@@ocaml.toplevel_printer]
 
-  (** {3 Bytestring conversion} *)
+  (** {3 Octets conversion} *)
 
-  (** [of_bytes_exn ipv6_octets] is the address represented
-      by [ipv6_octets]. Raises [Parse_error] if [ipv6_octets] is not a
-      valid representation of an IPv6 address. *)
-  val of_bytes_exn : string -> t
+  (** [of_octets_exn ?off ipv6_octets] is the address represented
+      by [ipv6_octets], starting from offset [off].
+      Raises {!Parse_error} if [ipv6_octets] is not a valid representation of
+      an IPv6 address. *)
+  val of_octets_exn : ?off:int -> string -> t
 
-  (** Same as [of_bytes_exn] but returns an option type instead of raising
+  (** Same as {!of_octets_exn} but returns an result type instead of raising
       an exception. *)
-  val of_bytes : string -> t option
+  val of_octets : ?off:int -> string -> (t, [> `Msg of string ]) result
 
-  (** Same as [of_bytes_exn] but takes an extra paramenter, the offset into
-      the bytes for reading. *)
-  val of_bytes_raw : string -> int -> t
+  (** [write_octets_exn ?off ipv6 b] writes 16 bytes that encode [ipv6] into [b] starting
+      from offset [off] within [b].  [b] must be at least [off+16] bytes long or
+      a {!Parse_error} exception will be raised. *)
+  val write_octets_exn : ?off:int -> t -> bytes -> unit
 
-  (** [to_bytes ipv6] is a string of length 16 encoding [ipv6]. *)
-  val to_bytes : t -> string
-
-  (** [to_bytes_raw ipv6 bytes offset] writes the 16 bytes encoding of [ipv6]
-      into [bytes] at offset [offset]. *)
-  val to_bytes_raw : t -> Bytes.t -> int -> unit
+  (** [write_octets ?off ipv6 b] writes 16 bytes that encode [ipv6] into [b] starting
+      from offset [off] within [b].  [b] must be at least [off+16] bytes long or
+      an error is returned. *)
+  val write_octets : ?off:int -> t -> bytes -> (unit, [> `Msg of string ]) result
+ 
+  (** [to_octets ipv6] returns the 16 bytes representing the [ipv6] octets. *)
+  val to_octets : t -> string
 
   (** {3 Int conversion} *)
 
   (** [of_int64 (ho, lo)] is the IPv6 address represented by two int64. *)
   val of_int64 : int64 * int64 -> t
+
   (** [to_int64 ipv6] is the 128-bit packed encoding of [ipv6]. *)
   val to_int64 : t -> int64 * int64
 
   (** [of_int32 (a, b, c, d)] is the IPv6 address represented by four int32. *)
   val of_int32 : int32 * int32 * int32 * int32 -> t
+
   (** [to_int32 ipv6] is the 128-bit packed encoding of [ipv6]. *)
   val to_int32 : t -> int32 * int32 * int32 * int32
 
   (** [of_int16 (a, b, c, d, e, f, g, h)] is the IPv6 address represented by
       eight 16-bit int. *)
   val of_int16 : int * int * int * int * int * int * int * int -> t
+
   (** [to_int16 ipv6] is the 128-bit packed encoding of [ipv6]. *)
   val to_int16 : t -> int * int * int * int * int * int * int * int
 
@@ -365,8 +400,12 @@ module V6 : sig
   (** {3 Host conversion} *)
 
   (** [to_domain_name ipv6] is the domain name label list for reverse
-      lookups of [ipv6]. This includes the [.ip6.arpa.] suffix. *)
-  val to_domain_name : t -> string list
+      lookups of [ipv6]. This includes the [.ip6.arpa] suffix. *)
+  val to_domain_name : t -> [ `host ] Domain_name.t
+
+  (** [of_domain_name name] is [Some t] if the [name] has an [.ip6.arpa]
+      suffix, and an IPv6 address prefixed. *)
+  val of_domain_name : 'a Domain_name.t -> t option
 
   (** {3 Common addresses} *)
 
@@ -393,12 +432,10 @@ module V6 : sig
 
   (** A module for manipulating IPv6 network prefixes. *)
   module Prefix : sig
-    type addr = t [@@deriving sexp]
+    type addr = t
 
     (** Type of a internet protocol subnet *)
-    type t [@@deriving sexp]
-
-    val compare : t -> t -> int
+    type t
 
     (** [mask n] is the pseudo-address of an [n] bit subnet mask. *)
     val mask : int -> addr
@@ -412,15 +449,15 @@ module V6 : sig
     val network_address : t -> addr -> addr
 
     (** [of_string_exn cidr] is the subnet prefix represented by the CIDR
-        string, [cidr]. Raises [Parse_error] if [cidr] is not a valid
+        string, [cidr]. Raises {!Parse_error} if [cidr] is not a valid
         representation of a CIDR notation routing prefix. *)
     val of_string_exn : string -> t
 
-    (** Same as [of_string_exn] but returns an option type instead of raising
+    (** Same as {!of_string_exn} but returns a result type instead of raising
         an exception. *)
-    val of_string : string -> t option
+    val of_string : string -> (t, [> `Msg of string ]) result
 
-    (** Same as [of_string_exn] but takes as an extra argument the offset
+    (** Same as {!of_string_exn} but takes as an extra argument the offset
         into the string for reading. *)
     val of_string_raw : string -> int ref -> t
 
@@ -428,18 +465,18 @@ module V6 : sig
         of [prefix], i.e. XXX:XX:X::XXX/XX. *)
     val to_string : t -> string
 
-    (** [pp_hum f prefix] outputs a human-readable representation of [prefix]
+    (** [pp f prefix] outputs a human-readable representation of [prefix]
         to the formatter [f]. *)
-    val pp_hum : Format.formatter -> t -> unit
+    val pp : Format.formatter -> t -> unit [@@ocaml.toplevel_printer]
 
     (** [of_address_string_exn cidr_addr] is the address and prefix
-        represented by [cidr_addr]. Raises [Parse_error] if [cidr_addr] is not
+        represented by [cidr_addr]. Raises {!Parse_error} if [cidr_addr] is not
         a valid representation of a CIDR-scoped address. *)
     val of_address_string_exn : string -> t * addr
 
-    (** Same as [of_address_string_exn] but returns an option type instead of
+    (** Same as {!of_address_string_exn} but returns an option type instead of
         raising an exception. *)
-    val of_address_string : string -> (t * addr) option
+    val of_address_string : string -> ((t * addr), [> `Msg of string]) result
 
     (** [to_address_string prefix addr] is the network address
         constructed from [prefix] and [addr]. *)
@@ -529,12 +566,10 @@ module V6 : sig
 end
 
 (** Type of either an IPv4 value or an IPv6 value *)
-type ('v4,'v6) v4v6 = V4 of 'v4 | V6 of 'v6 [@@deriving sexp]
+type ('v4,'v6) v4v6 = V4 of 'v4 | V6 of 'v6
 
 (** Type of any IP address *)
-type t = (V4.t,V6.t) v4v6 [@@deriving sexp]
-
-val compare : t -> t -> int
+type t = (V4.t,V6.t) v4v6
 
 (** [to_string addr] is the text string representation of [addr]. *)
 val to_string : t -> string
@@ -543,18 +578,18 @@ val to_string : t -> string
     [buf]. *)
 val to_buffer : Buffer.t -> t -> unit
 
-(** [pp_hum f ip] outputs a human-readable representation of [ip] to the
+(** [pp f ip] outputs a human-readable representation of [ip] to the
     formatter [f]. *)
-val pp_hum : Format.formatter -> t -> unit
+val pp : Format.formatter -> t -> unit [@@ocaml.toplevel_printer]
 
 (** [of_string_exn s] parses [s] as an IPv4 or IPv6 address.
-    Raises [Parse_error] if [s] is not a valid string representation of an IP
+    Raises {!Parse_error} if [s] is not a valid string representation of an IP
     address. *)
 val of_string_exn : string -> t
 
-(** Same as [of_string_exn] but returns an option type instead of raising an
+(** Same as {!of_string_exn} but returns a result type instead of raising an
     exception. *)
-val of_string : string -> t option
+val of_string : string -> (t, [> `Msg of string ]) result
 
 (** Same as [of_string_exn] but takes as an extra argument the offset into
     the string for reading. *)
@@ -595,16 +630,18 @@ val is_private : t -> bool
 val multicast_to_mac : t -> Macaddr.t
 
 (** [to_domain_name addr] is the domain name label list for reverse
-    lookups of [addr]. This includes the [.arpa.] suffix. *)
-val to_domain_name : t -> string list
+    lookups of [addr]. This includes the [.in-addr.arpa] or [.ip6.arpa] suffix. *)
+val to_domain_name : t -> [ `host ] Domain_name.t
+
+(** [of_domain_name name] is [Some t] if the [name] has an [.in-addr.arpa] or
+    [ip6.arpa] suffix, and an IP address prefixed. *)
+val of_domain_name : 'a Domain_name.t -> t option
 
 module Prefix : sig
-  type addr = t [@@deriving sexp]
+  type addr = t
 
   (** Type of a internet protocol subnet *)
-  type t = (V4.Prefix.t, V6.Prefix.t) v4v6 [@@deriving sexp]
-
-  val compare : t -> t -> int
+  type t = (V4.Prefix.t, V6.Prefix.t) v4v6
 
   (** [to_string subnet] is the text string representation of [subnet]. *)
   val to_string : t -> string
@@ -613,20 +650,20 @@ module Prefix : sig
       into [buf]. *)
   val to_buffer : Buffer.t -> t -> unit
 
-  (** [pp_hum f subnet] outputs a human-readable representation of [subnet]
+  (** [pp f subnet] outputs a human-readable representation of [subnet]
       to the formatter [f]. *)
-  val pp_hum : Format.formatter -> t -> unit
+  val pp : Format.formatter -> t -> unit [@@ocaml.toplevel_printer]
 
   (** [of_string_exn cidr] is the subnet prefix represented by the CIDR
-      string, [cidr]. Raises [Parse_error] if [cidr] is not a valid
+      string, [cidr]. Raises {!Parse_error} if [cidr] is not a valid
       representation of a CIDR notation routing prefix. *)
   val of_string_exn : string -> t
 
-  (** Same as [of_string_exn] but returns an option type instead of raising
+  (** Same as {!of_string_exn} but returns a result type instead of raising
       an exception. *)
-  val of_string     : string -> t option
+  val of_string : string -> (t, [> `Msg of string]) result
 
-  (** Same as [of_string_exn] but takes as an extra argument the offset
+  (** Same as {!of_string_exn} but takes as an extra argument the offset
       into the string for reading. *)
   val of_string_raw : string -> int ref -> t
 
