@@ -15,26 +15,28 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *)
 
-open Sexplib.Std
-
-exception Parse_error of string * string [@@deriving sexp]
+exception Parse_error of string * string
 
 let need_more x = Parse_error ("not enough data", x)
 
+let try_with_result fn a =
+  try Ok (fn a)
+  with Parse_error (msg, _) -> Error (`Msg ("Macaddr: " ^ msg))
+
 type t = Bytes.t (* length 6 only *)
 
 let compare = Bytes.compare
 
 (* Raw MAC address off the wire (network endian) *)
-let of_bytes_exn x =
+let of_octets_exn x =
   if String.length x <> 6
   then raise (Parse_error ("MAC is exactly 6 bytes", x))
   else Bytes.of_string x
 
-let of_bytes x = try Some (of_bytes_exn x) with _ -> None
+let of_octets x = try_with_result of_octets_exn x
 
 let int_of_hex_char c =
-  let c = int_of_char (Char.uppercase c) - 48 in
+  let c = int_of_char (Char.uppercase_ascii c) - 48 in
   if c > 9
   then if c > 16
     then c - 7 (* upper hex offset *)
@@ -88,13 +90,13 @@ let parse_sextuple s i =
       Bytes.set m 5 (Char.chr (parse_hex_int [] s i));
       (if !i - p <> 2 then raise (Parse_error ("hex pairs required",s)));
       m
-  with Invalid_argument "Char.chr" ->
+  with Invalid_argument _ ->
     raise (Parse_error ("address segment too large",s))
 
 (* Read a MAC address colon-separated string *)
 let of_string_exn x = parse_sextuple x (ref 0)
 
-let of_string x = try Some (of_string_exn x) with _ -> None
+let of_string x = try_with_result of_string_exn x
 
 let chri x i = Char.code (Bytes.get x i)
 
@@ -107,14 +109,10 @@ let to_string ?(sep=':') x =
     (chri x 4) sep
     (chri x 5)
 
-let to_bytes x = Bytes.to_string x
+let to_octets x = Bytes.to_string x
 
-let sexp_of_t m = Sexplib.Sexp.Atom (to_string m)
-
-let t_of_sexp m =
-  match m with
-  | Sexplib.Sexp.Atom m -> of_string_exn m
-  | _ -> raise (Failure "Macaddr.t: Unexpected non-atom in sexp")
+let pp ppf i =
+  Format.fprintf ppf "%s" (to_string i)
 
 let broadcast = Bytes.make 6 '\255'
 
