@@ -14,44 +14,50 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *)
 
-(** A library for manipulation of MAC address representations. *)
+(** A library for manipulation of MAC address representations.
 
-(** Raised when parsing of MAC address syntax fails. *)
-exception Parse_error of string * string [@@deriving sexp]
+    {e %%VERSION%% - {{:%%PKG_HOMEPAGE%% }homepage}} *)
+
+(** [Parse_error (err,packet)] is raised when parsing of the MAC
+    address syntax fails. [err] contains a human-readable error
+    and [packet] is the original octet list that failed to parse. *)
+exception Parse_error of string * string
 
 (** Type of the hardware address (MAC) of an ethernet interface. *)
-type t [@@deriving sexp]
+type t
 
-val compare : t -> t -> int
+(** {2 Functions converting MAC addresses to/from octets/string} *)
 
-(** Functions converting MAC addresses to bytes/string and vice
-    versa. *)
-
-(** [of_bytes_exn buf] is the hardware address extracted from
+(** [of_octets_exn buf] is the hardware address extracted from
     [buf]. Raises [Parse_error] if [buf] has not length 6. *)
-val of_bytes_exn : string -> t
+val of_octets_exn : string -> t
 
-(** Same as above but returns an option type instead of raising an
-    exception. *)
-val of_bytes : string -> t option
+(** Same as {!of_octets_exn} but returns a result type instead of
+    raising an exception. *)
+val of_octets : string -> (t, [> `Msg of string]) result
 
-(** [of_string_exn mac_string] is the hardware address represented by
-    [mac_string]. Raises [Parse_error] if [mac_string] is not a
+(** [of_string_exn mac_string] is the human-readable hardware address represented by
+    [mac_string]. Raises {!Parse_error} if [mac_string] is not a
     valid representation of a MAC address. *)
 val of_string_exn : string -> t
 
-(** Same as above but returns an option type instead of raising an
+(** Same as {!of_string_exn} but returns a result type instead of raising an
     exception. *)
-val of_string : string -> t option
+val of_string : string ->  (t, [> `Msg of string]) result
 
-(** [to_bytes mac_addr] is a string of size 6 encoding [mac_addr]. *)
-val to_bytes : t -> string
+(** [to_octets mac_addr] is a string of size 6 encoding [mac_addr] as a
+    sequence of bytes. *)
+val to_octets : t -> string
 
 (** [to_string ?(sep=':') mac_addr] is the [sep]-separated string representation
-    of [mac_addr], i.e. xx:xx:xx:xx:xx:xx. *)
+    of [mac_addr], i.e. [xx:xx:xx:xx:xx:xx]. *)
 val to_string : ?sep:char -> t -> string
 
-(** [broadcast] is ff:ff:ff:ff:ff:ff. *)
+(** [pp f mac_addr] outputs a human-readable representation of [mac_addr] to
+    the formatter [f]. *)
+val pp : Format.formatter -> t -> unit [@@ocaml.toplevel_printer]
+
+(** [broadcast] is [ff:ff:ff:ff:ff:ff]. *)
 val broadcast : t
 
 (** [make_local bytegen] creates a unicast, locally administered MAC
