@@ -20,7 +20,6 @@ open Ipaddr
 
 let error s msg = s, Parse_error (msg,s)
 let need_more s = error s "not enough data"
-let too_much s  = error s "too much data"
 let bad_char i s =
   error s (Printf.sprintf "invalid character '%c' at %d" s.[i] i)
 
@@ -44,7 +43,7 @@ module Test_v4 = struct
       let os = V4.of_string_exn addr in
       let ts = V4.to_string os in
       assert_equal ~msg:addr ts rt;
-      let os = V4.t_of_sexp (V4.sexp_of_t os) in
+      let os = Ipaddr_sexp.(V4.t_of_sexp (V4.sexp_of_t os)) in
       let ts = V4.to_string os in
       assert_equal ~msg:addr ts rt;
     ) addrs
@@ -89,16 +88,15 @@ module Test_v4 = struct
   let test_bytes_rt () =
     let addr = "\254\099\003\128" in
     assert_equal ~msg:(String.escaped addr)
-      V4.(to_bytes (of_bytes_exn addr)) addr
+      V4.(to_octets (of_octets_exn addr)) addr
 
   let test_bytes_rt_bad () =
     let addrs = [
       need_more "\254\099\003";
-      too_much "\254\099\003\128\001";
     ] in
     List.iter (fun (addr,exn) ->
       assert_raises ~msg:(String.escaped addr) exn
-        (fun () -> V4.of_bytes_exn addr)
+        (fun () -> V4.of_octets_exn addr)
     ) addrs
 
   let test_int32_rt () =
@@ -117,7 +115,7 @@ module Test_v4 = struct
       let os = V4.Prefix.of_string_exn subnet in
       let ts = V4.Prefix.to_string os in
       assert_equal ~msg:subnet ts rt;
-      let os = V4.Prefix.(t_of_sexp (sexp_of_t os)) in
+      let os = Ipaddr_sexp.(V4.Prefix.(t_of_sexp (sexp_of_t os))) in
       let ts = V4.Prefix.to_string os in
       assert_equal ~msg:subnet ts rt;
     ) subnets
@@ -273,7 +271,7 @@ module Test_v4 = struct
     assert_equal ~msg:"localhost" true V4.(Prefix.(mem localhost loopback))
 
   let test_multicast_mac () =
-    let ip = V4.of_bytes_exn "\xff\xbf\x9f\x8f" in
+    let ip = V4.of_octets_exn "\xff\xbf\x9f\x8f" in
     let multicast = V4.Prefix.(network_address multicast ip) in
     let unicast_mac_str   = Macaddr.to_string (V4.multicast_to_mac ip) in
     let multicast_mac_str = Macaddr.to_string (V4.multicast_to_mac multicast) in
@@ -285,9 +283,26 @@ module Test_v4 = struct
 
   let test_domain_name () =
     let ip = V4.of_string_exn "128.64.32.16" in
-    let name = "16.32.64.128.in-addr.arpa." in
-    assert_equal ~msg:"domain_name"
-      (String.concat "." (V4.to_domain_name ip)) name
+    let name =
+      Domain_name.(host_exn (of_string_exn "16.32.64.128.in-addr.arpa"))
+    in
+    assert_equal ~cmp:Domain_name.equal ~msg:"to_domain_name"
+      (V4.to_domain_name ip) name ;
+    assert_equal ~msg:"of_domain_name" (V4.of_domain_name name) (Some ip)
+
+  let test_cstruct_rt () =
+    let addr = "\254\099\003\128" in
+    assert_equal ~msg:(String.escaped addr)
+      (Cstruct.to_string Ipaddr_cstruct.V4.(to_cstruct (of_cstruct_exn (Cstruct.of_string addr)))) addr
+
+  let test_cstruct_rt_bad () =
+    let addrs = [
+      need_more "\254\099\003";
+    ] in
+    List.iter (fun (addr,exn) ->
+      assert_raises ~msg:(String.escaped addr) exn
+        (fun () -> Ipaddr_cstruct.V4.of_cstruct_exn (Cstruct.of_string addr))
+    ) addrs
 
   let suite = "Test V4" >::: [
     "string_rt"            >:: test_string_rt;
@@ -296,6 +311,8 @@ module Test_v4 = struct
     "string_raw_rt_bad"    >:: test_string_raw_rt_bad;
     "bytes_rt"             >:: test_bytes_rt;
     "bytes_rt_bad"         >:: test_bytes_rt_bad;
+    "cstruct_rt"           >:: test_cstruct_rt;
+    "cstruct_rt_bad"       >:: test_cstruct_rt_bad;
     "int32_rt"             >:: test_int32_rt;
     "prefix_string_rt"     >:: test_prefix_string_rt;
     "prefix_string_rt_bad" >:: test_prefix_string_rt_bad;
@@ -332,7 +349,7 @@ module Test_v6 = struct
       let os = V6.of_string_exn addr in
       let ts = V6.to_string os in
       assert_equal ~msg:(addr^" <> "^rt^" ("^ts^")") ts rt;
-      let os = V6.t_of_sexp (V6.sexp_of_t os) in
+      let os = Ipaddr_sexp.(V6.t_of_sexp (V6.sexp_of_t os)) in
       let ts = V6.to_string os in
       assert_equal ~msg:(addr^" <> "^rt^" ("^ts^")") ts rt;
     ) addrs
@@ -340,6 +357,7 @@ module Test_v6 = struct
   let test_string_rt_bad () =
     let addrs = [
       need_more "[";
+      need_more "[:";
       need_more "[]"; (* ? *)
       need_more ":";
       need_more "[::";
@@ -401,18 +419,32 @@ module Test_v6 = struct
     let addr =
       "\000\000\000\000\000\000\000\000\000\000\255\255\192\168\000\001"
     in
-    let v6 = V6.of_bytes_exn addr in
-    assert_equal ~msg:(String.escaped addr) V6.(to_bytes v6) addr
+    let v6 = V6.of_octets_exn addr in
+    assert_equal ~msg:(String.escaped addr) V6.(to_octets v6) addr
 
   let test_bytes_rt_bad () =
     let addrs = [
       need_more "\000\000\000\000\000\000\000\000\000\000\255\255\192\168\001";
-      too_much
-        "\000\000\000\000\000\000\000\000\000\000\255\255\192\168\000\000\001";
     ] in
     List.iter (fun (addr,exn) ->
       assert_raises ~msg:(String.escaped addr) exn
-        (fun () -> V6.of_bytes_exn addr)
+        (fun () -> V6.of_octets_exn addr)
+    ) addrs
+
+  let test_cstruct_rt () =
+    let addr =
+      "\000\000\000\000\000\000\000\000\000\000\255\255\192\168\000\001"
+    in
+    let v6 = Ipaddr_cstruct.V6.of_cstruct_exn (Cstruct.of_string addr) in
+    assert_equal ~msg:(String.escaped addr) (Cstruct.to_string Ipaddr_cstruct.V6.(to_cstruct v6)) addr
+
+  let test_cstruct_rt_bad () =
+    let addrs = [
+      need_more "\000\000\000\000\000\000\000\000\000\000\255\255\192\168\001";
+    ] in
+    List.iter (fun (addr,exn) ->
+      assert_raises ~msg:(String.escaped addr) exn
+        (fun () -> Ipaddr_cstruct.V6.of_cstruct_exn (Cstruct.of_string addr))
     ) addrs
 
   let test_int32_rt () =
@@ -437,7 +469,7 @@ module Test_v6 = struct
       let os = V6.Prefix.of_string_exn subnet in
       let ts = V6.Prefix.to_string os in
       assert_equal ~msg:subnet ts rt;
-      let os = V6.Prefix.(t_of_sexp (sexp_of_t os)) in
+      let os = Ipaddr_sexp.(V6.Prefix.(t_of_sexp (sexp_of_t os))) in
       let ts = V6.Prefix.to_string os in
       assert_equal ~msg:subnet ts rt;
     ) subnets
@@ -601,10 +633,12 @@ module Test_v6 = struct
   let test_domain_name () =
     let ip = V6.of_string_exn "2a00:1450:4009:800::200e" in
     let name =
-      "e.0.0.2.0.0.0.0.0.0.0.0.0.0.0.0.0.0.8.0.9.0.0.4.0.5.4.1.0.0.a.2.ip6.arpa."
+      "e.0.0.2.0.0.0.0.0.0.0.0.0.0.0.0.0.0.8.0.9.0.0.4.0.5.4.1.0.0.a.2.ip6.arpa"
     in
-    assert_equal ~msg:"domain_name"
-      (String.concat "." (V6.to_domain_name ip)) name
+    let name = Domain_name.(host_exn (of_string_exn name)) in
+    assert_equal ~cmp:Domain_name.equal ~msg:"to_domain_name"
+      (V6.to_domain_name ip) name ;
+    assert_equal ~msg:"of_domain_name" (V6.of_domain_name name) (Some ip)
 
   let test_link_address_of_mac () =
     let mac = Macaddr.of_string_exn "34-56-78-9A-BC-DE" in
@@ -620,6 +654,8 @@ module Test_v6 = struct
     "string_raw_rt_bad"    >:: test_string_raw_rt_bad;
     "bytes_rt"             >:: test_bytes_rt;
     "bytes_rt_bad"         >:: test_bytes_rt_bad;
+    "cstruct_rt"           >:: test_cstruct_rt;
+    "cstruct_rt_bad"       >:: test_cstruct_rt_bad;
     "int32_rt"             >:: test_int32_rt;
     "prefix_string_rt"     >:: test_prefix_string_rt;
     "prefix_string_rt_bad" >:: test_prefix_string_rt_bad;
